/************************************************************************************//**
* \file         memdrv.c
* \brief        Memory driver source file.
* \internal
*----------------------------------------------------------------------------------------
*                          C O P Y R I G H T
*----------------------------------------------------------------------------------------
*   Copyright (c) 2024  by Feaser    http://www.feaser.com    All rights reserved
*
*----------------------------------------------------------------------------------------
*                            L I C E N S E
*----------------------------------------------------------------------------------------
*
* SPDX-License-Identifier: MIT
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
*
* \endinternal
****************************************************************************************/

/****************************************************************************************
* Include files
****************************************************************************************/
#include "boot.h"                                /* bootloader generic header          */
#include "memdrv.h"                              /* memory device driver header        */


/* This source file implements the driver framework for an additional memory device. It
 * is interfaced to the bootloader's non-volatile memory (NVM) module, through
 * the NvmXxxHook() functions located in "hooks.c". The configuration macro
 * BOOT_NVM_HOOKS_ENABLE was set to 1 to activate these hook-functions.
 *
 * In a nutshell, you just need to implement the low-level memory erase and program
 * functionality in this source-file:
 *
 *   - MemDrvInit()
 *   - MemDrvWrite()
 *   - MemDrvErase()
 *   - MemDrvGetSize()
 *   - MemDrvGetBaseAddress()
 *   - MemDrvDone()
 *
 * Afterwards, the bootloader can reprogram data in this additional memory device, during
 * a firmware update.
 */


/****************************************************************************************
* Macro definitions
****************************************************************************************/
/* TODO ##MemDrv Set the total size in bytes of the memory device. */
#define MEM_DRV_DEVICE_TOTAL_SIZE      (16U * 1024U)

/* TODO ##MemDrv Set the base address of the memory device in the memory map. */
#define MEM_DRV_DEVICE_BASE_ADDR       (0x08080000UL)


/************************************************************************************//**
** \brief     Initializes the memory device driver.
** \return    none.
**
****************************************************************************************/
void MemDrvInit(void)
{
  /* TODO ##MemDrv Initialize your memory device driver. For example enabling
   *      and configuring the QSPI peripheral.
   */
} /*** end of MemDrvInit ***/


/************************************************************************************//**
** \brief     Write a certain amount of bytes to the memory device, starting at the
**            specified base address.
** \param     address The base address for the write operation.
** \param     len The number of bytes to write.
** \param     data Pointer to byte array where the data is read from.
** \return    BLT_TRUE if successful, BLT_FALSE otherwise.
**
****************************************************************************************/
blt_bool MemDrvWrite(blt_addr address, blt_int32u len, blt_int8u const * data)
{
  blt_bool result = BLT_TRUE;

  /* TODO ##MemDrv Program all the data byte at the specified memory location
   *      of your memory device. Set result to BLT_FALSE upon error detection.
   */

  /* Give the result back to the caller. */
  return result;
} /*** end of MemDrvWrite ***/


/************************************************************************************//**
** \brief     Erase a certain amount of bytes from the memory device, starting at the
**            specified base address.
** \param     address The base address for the erase operation.
** \param     len The number of bytes to erase.
** \return    BLT_TRUE if successful, BLT_FALSE otherwise.
**
****************************************************************************************/
blt_bool MemDrvErase(blt_addr address, blt_int32u len)
{
  blt_bool result = BLT_TRUE;

  /* TODO ##MemDrv Erase the specified range of memory. Set result to BLT_FALSE
   *      upon error detection. If your memory device automatically erases a
   *      byte before programming a new one, there is no need to implement
   *      anything here.
   */

  /* Give the result back to the caller. */
  return result;
} /*** end of MemDrvErase ***/


/************************************************************************************//**
** \brief     Finalizes the memory device driver operations.
** \return    BLT_TRUE is succesful, BLT_FALSE otherwise.
**
****************************************************************************************/
blt_bool MemDrvDone(void)
{
  blt_bool result = BLT_TRUE;

  /* TODO ##MemDrv This function is called at the end of a firmware update. Set
   *      result to BLT_FALSE upon error detection. You can use it to finalize
   *      the memory device erase/program operations, if applicable. For example
   *      if data is first buffered in RAM blocks, before actually writing to
   *      the memory device, there might still be data pending to be written to
   *      the device at the end of the firmware update. This can be done in
   *      this function.
   */

  /* Give the result back to the caller. */
  return result;
} /*** end of MemDrvDone ***/


/************************************************************************************//**
** \brief     Getter to read the total size of the memory device.
** \return    Total size in bytes of the memory device.
**
****************************************************************************************/
blt_int32u MemDrvGetSize(void)
{
  return MEM_DRV_DEVICE_TOTAL_SIZE;
} /*** end of MemDrvGetSize ***/


/************************************************************************************//**
** \brief     Getter to read the base address of the memory device in the memory map.
** \return    Base address of the memory device.
**
****************************************************************************************/
blt_addr MemDrvGetBaseAddress(void)
{
  return MEM_DRV_DEVICE_BASE_ADDR;
} /*** end of MemDrvGetBaseAddress ***/

/*********************************** end of memdrv.c ***********************************/
